/* FBUG68K. From sources from Holm Tiffe August 2023
            From Motorola Freeware site circa 1994

   Dragged into the 21st Century by WRM August 2023 - WIP
   There is also dBUG(PPC)
   FBUG68K -> dBUG68K -> dBUGPPC (I think)
   dBUGPPC supports MC68230, MC68681, MC68901, NS16550
   Some of that code incorporated here

   All code refactored to reflect modern usage.

   function()          -->  int function(int arg1, int arg2)
   int arg1;
   int arg2;
   {
     ..
     return(0)
   }

   And so forth, and so on.

   The target system memory map is defined in targetsys.h but also
   pay attention to the .ld files because ld does not know about
   targetsys.h

   0x00000000 - 0x000003FF : USERVBRLOC -- 68000 vectors
   0x00000400              : SYSRAMLOC
   0x00001400              : VBRLOC
   0x00001F00              : USPLOC
   0x00002000              : MSPLOC
   0x00002400              : ISPLOC * defines for monitor  *

   0x00004000              : USERPCLOC * defines for usercode *


*/

/*
  asmstartport.s has __start which sets up the basics and then
  calls main()
*/

/* ************************************************************** */

/*  (WRM note the order described below is obviously wrong)
This is the initialization routine. The cold start will come here. 
First the duart is initialized, then the register table for the mpu
registers. Next the symbol, followed by the break, table are
initialized. The mainloop is called and the monitor is off and
running.
*/
/* ************************************************************** */

#include "general.h"

void main(void)
{
  extern struct breakelem brtable[];
  extern struct symbelem symb[];
  extern struct regelem mpu[];
  extern int asmhandler;
  register int i,j,k,l;

  if (get32(VBRLOC) == 1)
  {
    put32(VBRLOC,0);
    i = 0;
    mpu[i].value = USERPCLOC;
    mpu[i++].name = p_c;
    mpu[i].value = SRSTART;
    mpu[i++].name = s_r;
#if(DEVICE>=68020)
    mpu[i++].name = "=";
#endif
    mpu[i].value = USERUSPLOC;
    mpu[i++].name = usp;
#if(DEVICE<68020)
    mpu[i++].name = "=";
#endif
    mpu[i].value = USERMSPLOC;
    mpu[i++].name = msp;
#if(DEVICE<68020)
    mpu[i].value = USERSSPLOC;
    mpu[i++].name = ssp;
#else
    mpu[i].value = USERISPLOC;
    mpu[i++].name = isp;
#endif
    mpu[i].value = USERVBRLOC;
    mpu[i++].name = vbr;
    j = i;
    mpu[i++].name = cac;
    mpu[i++].name = caa;
    mpu[i++].name = sfc;
    mpu[i++].name = dfc;
    mpu[i++].name = "=";
    mpu[i++].name = d_0;
    mpu[i++].name = d_1;
    mpu[i++].name = d_2;
    mpu[i++].name = d_3;
    mpu[i++].name = d_4;
    mpu[i++].name = d_5;
    mpu[i++].name = d_6;
    mpu[i++].name = d_7;
    mpu[i++].name = "=";
    mpu[i++].name = a_0;
    mpu[i++].name = a_1;
    mpu[i++].name = a_2;
    mpu[i++].name = a_3;
    mpu[i++].name = a_4;
    mpu[i++].name = a_5;
    mpu[i++].name = a_6;
    k=i;
#if(DEVICE<68020)
    mpu[i].value = USERSSPLOC;
#else
    mpu[i].value = USERISPLOC;
#endif
    mpu[i++].name = a_7;
#if(DEVICE==68030)
    mpu[i++].name = "=";
    mpu[i++].name = "=";
    mpu[i].value = 1;
    mpu[i++].name = crp;
    mpu[i++].name = "crpl";
    mpu[i].value = 1;
    mpu[i++].name = srp;
    mpu[i++].name = "srpl";
    mpu[i++].name = "=";
    mpu[i++].name = tc;
    mpu[i++].name = tt_0;
    mpu[i++].name = tt_1;
    mpu[i++].name = mmusr;
#endif
#if(DEVICE==68040)
    mpu[i++].name = "=";
    mpu[i++].name = "=";
    mpu[i].value = 1;
    mpu[i++].name = urp;
    mpu[i++].name = "urpl";
    mpu[i].value = 1;
    mpu[i++].name = srp;
    mpu[i++].name = "srpl";
    mpu[i++].name = tc;
    mpu[i++].name = "=";
    mpu[i++].name = dtt_0;
    mpu[i++].name = dtt_1;
    mpu[i++].name = itt_0;
    mpu[i++].name = itt_1;
    mpu[i++].name = mmusr;
#endif
    l = i;

#if(DEVICE==68040 || COPROCESSOR==TRUE)
    mpu[i++].name = "=";
    mpu[i++].name = "=";
    mpu[i].value = 0;
    mpu[i++].name = "FPCR ";
    mpu[i].value = 0;
    mpu[i++].name = "FPSR ";
    mpu[i].value = 0;
    mpu[i++].name = "FPIAR ";
    l = i;
    mpu[i++].name = "=";
    mpu[i++].name = "FP0  ";
    mpu[i++].name = "fp01 ";
    mpu[i++].name = "fp02 ";
    mpu[i++].name = "FP1  ";
    mpu[i++].name = "fp11 ";
    mpu[i++].name = "fp12 ";
    mpu[i++].name = "FP2  ";
    mpu[i++].name = "fp21 ";
    mpu[i++].name = "fp22 ";
    mpu[i++].name = "FP3  ";
    mpu[i++].name = "fp31 ";
    mpu[i++].name = "fp32 ";
    mpu[i++].name = "FP4  ";
    mpu[i++].name = "fp41 ";
    mpu[i++].name = "fp42 ";
    mpu[i++].name = "FP5  ";
    mpu[i++].name = "fp51 ";
    mpu[i++].name = "fp52 ";
    mpu[i++].name = "FP6  ";
    mpu[i++].name = "fp61 ";
    mpu[i++].name = "fp62 ";
    mpu[i++].name = "FP7  ";
    mpu[i++].name = "fp71 ";
    mpu[i++].name = "fp72 ";
#endif
    mpu[i].name = LASTCMD; // 0

    /* very complex way of setting everything that's not zero and not
       already initialised to 0. Probably easier to set everything to
       zero at the beginning but maybe we don't know the size because
       of different processors? Can't be a bigger kludge than this.
    */
    for(i=j; mpu[i].name != LASTCMD; i++)
    {
      if(i!=k && i!=k+3 && i!=k+5 && i<=l)
      {
        mpu[i].value = 0;
      }
    }

    /* Clear symbol table, set symb[0] = "/rom", ROMLOC */
    for (i = 0; ROMSYMB[i] != ENDSTR; i++)
    {
      symb[0].name[i] = ROMSYMB[i];
    }
    symb[0].value = ROMLOC;
    for (i = 1; i < MAXSYMBOL; i++)
    {
      symb[i].name[0] = ENDSTR;
      symb[i].value = 0;
    }

    /* breakpoint table */
    for (i = 0,j = 0;i < MAXBR;i++)
    {
      brtable[i].count = -1;
      brtable[i].tempcount = 0;
      brtable[i].address = 0; brtable[i].inst1 = 0;
      brtable[i].inst2 = 0;
    }

    /* The default interrupt handler is in asmhandler.s */
    for (i = 0x8;i < 0x3fc;i = i + 4)
    {
      put32(USERVBRLOC + i, (int)&asmhandler);
      put32(VBRLOC + i, (int)&asmhandler);
    }

    /* Initialise MC68681 DUART */
    /*
    targetsys.h:#define DUARTALOC 0xffe02000  * MC68681 duart channel a
    targetsys.h:#define TERMINAL DUARTALOC    * location of terminal (i.e. screen)
    targetsys.h:#define DUARTBLOC 0xffe02008  * MC68681 duart channel b
    targetsys.h:#define HOST DUARTBLOC        * location of host computer port
    */
    put8(HOST + CRX,0x30);
    put8(HOST + CRX,0x20);
    put8(HOST + CRX,0x10);
    put8(HOST + CSRX,0xbb);
    put8(HOST + MR1X,0x13);
    put8(HOST + MR2X,0x07);
    put8(HOST + CRX,0x05);
    put8(TERMINAL + CRX,0x30);
    put8(TERMINAL + CRX,0x20);
    put8(TERMINAL + CRX,0x10);
    put8(TERMINAL + IVR,0x0f);
    put8(TERMINAL + IMR,0x00);
    put8(TERMINAL + ACR,0x00);
    put8(TERMINAL + CTUR,0x00);
    put8(TERMINAL + CTLR,0x02);
    put8(TERMINAL + OPCR,0x00);
    put8(TERMINAL + CSRX,0xbb);
    put8(TERMINAL + MR1X,0x13);
    put8(TERMINAL + MR2X,0x07);
    put8(TERMINAL + CRX,0x05);
  }

  /* int print(...) is in printport.c */

  print("Copyright Motorola Inc. 1989   All Rights Reserved\n");
  print("Fbug68 Monitor/Debugger Version 1.1 - 9/28/89 \n");
  mainloop();
}

/* *************************************************************** */

