OUTPUT_ARCH(m68k)
OUTPUT_FORMAT("binary")

/* WRM for my homebrew 68000 */

MEMORY
{
    /* Operating system uses 128K of the 1M of static RAM */
    RAM (rwx) : ORIGIN = 0x00000400, LENGTH = 63K /* after the 1k byte of vectors */
    ROM (rx) : ORIGIN = 0x00010000, LENGTH = 64K
}

/* stack location */
stack_size = 0x1000;

_stack_start = 0x0000F000;
_stack_end = _stack_start - stack_size;

/* ram location */
_ram_start = ORIGIN(RAM);
_ram_end = ORIGIN(RAM)+LENGTH(RAM);

SECTIONS
{
	.text :
	{
    /* Vectors at start of ROM, gets mapped to $00000000 at reset */
	  LONG (_stack_start) 	/* System Stack Pointer */
	  LONG (__start)		    /* Program start code */

        . = ALIGN(0x4);
		__s_text = . ;
        *(.text)                 /* .text sections (asm code only because of -ffunction-sections to gcc) */
        *(.text.startup.main)    /* GCC puts main() (in coldport.c) in .text.startup, -ffunction-sections adds .main */
        *(.text*)                /* .text* sections (code) */
		CONSTRUCTORS

        *(.rodata .rodata.*)

        . = ALIGN(0x4);
        *(.gcc_except_table)

        . = ALIGN(0x4);
        *(.eh_frame)

        KEEP (*(.init))
        KEEP (*(.fini))

		__e_text = . ;

	} > ROM

    .rodata :
    {
        . = ALIGN(4);
        *(.rodata)         /* .rodata sections (constants, strings, etc.) */
        *(.rodata*)        /* .rodata* sections (constants, strings, etc.) */
        . = ALIGN(4);
    } > ROM

    .bss :
	{
        . = ALIGN(0x4);
		__s_bss = . ; /* We should be able */
		*(.bss)
		*(COMMON)
        . = ALIGN(0x4);
		__e_bss = . ; /* to clear the bss. */
	} > RAM

    .data :
    {
        . = ALIGN(0x4);
		__s_data = . ; /* Symbols, to know where to */
        *(.got.plt) *(.got)
        *(.shdata)
        *(.data .data.*)
		__e_data = . ; /* copy the data.            */
        . = ALIGN(0x4);
    } > RAM AT > ROM
}
